import React from "react";
import {
  LOADING,
  PRE_GAME,
  PRE_ROUND,
  IN_ROUND,
  POST_ROUND,
  POST_GAME,
  ERROR,
} from "./game-state.enum";
import PreGame from '../pre-game.component';
import PreRound from '../pre-round.component';
import GamePanel from "../game-panel.component";
import RoundSummary from '../round-summary.component';
import PlayerScores from "../player-scores.component";
import { 
  getGoogleApiKey, 
  createGame, 
  gameInfo, 
  joinGame, 
  getGuesses, 
  sendGuess 
} from "../../services/ggsh.service";

// TODO I want to break this down into some smaller container components that can handle making the API calls

class Game extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      googleApiKey: null,
      gameState: LOADING,
      playerName: null,
      gameId: null,
      currentRound: null,
      targetPoint: null,
      selectedPoint: null,
      lastScore: null,
      totalScore: null,
      players: null,
      roundTimer: null,
    }
  }
  
  // TODO error handling throughout - at the moment it assumes all calls always succeed
  
  async componentDidMount() {
    const googleApiKey = await getGoogleApiKey();
    this.setState({ googleApiKey, gameState: PRE_GAME });
  }
  
  async handleCreateGame() {
    this.setState({ gameState: LOADING });
    const { playerName } = this.state;
    const gameId = await createGame(playerName, 300);
    this.setState({ gameState: PRE_ROUND, gameId });
  }
  
  async handleJoinGame() {
    this.setState({ gameState: LOADING });
    const { gameId, playerName } = this.state;
    await joinGame(gameId, playerName);
    this.setState({ gameState: PRE_ROUND });
  }
  
  async updateRoundState() {
    this.setState({ gameState: LOADING })
    const { gameId, playerName } = this.state;
    const { currentRound } = await getGuesses(gameId, playerName);
    const { coords, players, timer } = await gameInfo(gameId);
    if (currentRound) {
      const targetPoint = coords[currentRound];
      this.setState({
        gameState: IN_ROUND,
        currentRound,
        targetPoint,
        selectedPoint: null,
        players,
        roundTimer: timer,
      });
    } else {
      this.setState({ gameState: POST_GAME, players });
    }
  }
  
  async handleSubmitGuess() {
    this.setState({ gameState: LOADING });
    const {
      gameId,
      playerName,
      currentRound,
      selectedPoint
    } = this.state;
    const { score, totalScore } = await sendGuess(gameId, playerName, currentRound, selectedPoint || { timeout: true });
    this.setState({
      gameState: POST_ROUND,
      lastScore: score,
      totalScore
    });
  }
  
  renderLoading() {
    return <p>Loading...</p>
  }
  
  renderPreGame() {
    const { gameId, playerName } = this.state;
    return <PreGame
    onCreateGame={() => this.handleCreateGame()}
    onJoinGame={() => this.handleJoinGame()}
    onChangeGameId={({ target }) => this.setState({gameId: target.value.trim()})}
    onChangePlayerName={({ target }) => this.setState({playerName: target.value.trim()})}
    gameId={gameId || ""}
    playerName={playerName || ""}
    />
  }
  
  renderPreRound() {
    const { gameId, playerName } = this.state;
    return <PreRound
    gameId={gameId}
    playerName={playerName}
    onStart={() => this.updateRoundState()}
    />
  }
  
  renderInRound() {
    const { googleApiKey, targetPoint, selectedPoint, roundTimer } = this.state;
    return <GamePanel
    googleApiKey={googleApiKey}
    onSelectPoint={latLng => this.setState({selectedPoint: latLng})}
    onSubmitGuess={() => this.handleSubmitGuess()}
    streetViewPoint={targetPoint}
    selectedPoint={selectedPoint}
    roundSeconds={roundTimer}
    onTimeout={() => this.handleSubmitGuess()}
    />
  }
  
  renderPostRound() {
    const {
      googleApiKey,
      currentRound,
      lastScore,
      totalScore,
      targetPoint,
      selectedPoint,
    } = this.state;
    return <RoundSummary
    googleApiKey={googleApiKey}
    roundNum={currentRound}
    score={lastScore}
    totalScore={totalScore}
    onAdvanceState={() => this.updateRoundState()}
    buttonText={currentRound === "5" ? "View Summary" : "Next Round"}
    targetPoint={targetPoint}
    selectedPoint={selectedPoint}
    />
  }
  
  renderPostGame() {
    const { players } = this.state;
    return <PlayerScores
    players={players}
    onReturnToStart={() => this.setState({ gameState: PRE_GAME })}
    />
  }
  
  renderError() {
    return <p>Application encountered unrecoverable error, please refresh the page.</p>
  }
  
  render() {
    const { gameState } = this.state;
    
    switch (gameState) {
      case LOADING:
      return this.renderLoading();
      case PRE_GAME:
      return this.renderPreGame();
      case PRE_ROUND:
      return this.renderPreRound();
      case IN_ROUND:
      return this.renderInRound();
      case POST_ROUND:
      return this.renderPostRound();
      case POST_GAME:
      return this.renderPostGame();
      case ERROR:
      return this.renderError();
      default:
      this.setState({ gameState: ERROR });
      return <p>Application state is inconsistent, please refresh and rejoin your previous game.</p>
    }
  }
}

export default Game;
